# Author: Michael Lissner
# Date created: 2013-06-05

from datetime import date

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.set_url(date.today().year)
        self.back_scrape_iterable = range(2009, 2013)

    def _get_download_urls(self):
        return self.get_cell_data(2, '//a/@href')

    def _get_case_names(self):
        return self.get_cell_data(2, '//a/text()')

    def _get_case_dates(self):
        return [convert_date_string(ds) for ds in self.get_cell_data(1, '//text()')]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_docket_numbers(self):
        return self.get_cell_data(4, '//text()')

    def _get_neutral_citations(self):
        return self.get_cell_data(3, '//text()')

    def _download_backwards(self, year):
        self.set_url(year)
        self.html = self._download()

    def get_cell_data(self, cell_number, sub_path=False):
        path = '//table[@id="GridView1"]/tr/td[%d]' % cell_number
        if sub_path:
            path += sub_path
        return self.html.xpath(path)

    def set_url(self, year):
        self.url = 'http://www.nmcompcomm.us/nmcases/NMARYear.aspx?db=scr&y1=%s&y2=%s' % (year, year)



"""Scraper for Wyoming Supreme Court
CourtID: wyo
Court Short Name: Wyo.
History:
 - 2014-07-02: mlr: Created new version when court got new website.
 - 2015-07-06: m4h7: Updated to use JSON!
 - 2016-06-09: arderyp: Updated because json endpoint moved and was changed
"""

import re
from datetime import datetime, date

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.base_url = 'http://www.courts.state.wy.us'
        self.url = 'https://opinions.courts.state.wy.us/Home/GetOpinions?StartDate=1%2F1%2F' + str(date.today().year)

    def _get_case_names(self):
        return ['%s v. %s' % (opinion['Appellant'], opinion['Appellee']) for opinion in self.html]

    def _get_download_urls(self):
        download_urls = []
        for record in self.html:
            pdf_file_name = record['DocumentName']
            if pdf_file_name[:5] == '../..':
                pdf_file_name = pdf_file_name[5:]
            url = self.base_url + '/Documents/Opinions/' + pdf_file_name
            download_urls.append(url)
        return download_urls

    def _get_case_dates(self):
        case_dates = []
        date_re = re.compile(r"^/Date\((\d+)\)/$")
        for record in self.html:
            match = date_re.match(record['date_heard'])
            if match:
               timestamp = int(match.group(1)) / 1000
               case_dates.append(datetime.fromtimestamp(timestamp).date())
        return case_dates

    def _get_docket_numbers(self):
        return [opinion['DocketNumber'] for opinion in self.html]

    def _get_neutral_citations(self):
        return [opinion['OpinionID'] for opinion in self.html]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)



import difflib
import string

import re

from .string_utils import harmonize

STOP_WORDS = (
    'a|an|and|as|at|but|by|en|etc|for|if|in|is|of|on|or|the|to|v\.?|via|'
    'vs\.?|et|al|appellants?|defendants?|administrator|'
    'plaintiffs?|error|others|against|ex|parte|complainants?|original|'
    'claimants?|devisee|executrix|executor'
)
STOP_WORDS_RE = re.compile(r'^(%s)$' % STOP_WORDS)


def normalize_phrase(phrase):
    """Clean up words or phrases before sending them to be compared.

     - Harmonize things like United States, USA, etc.
     - Normalize to lower case.
     - Remove punctuation.
     - Remove stop words.
    """
    phrase = harmonize(phrase)
    phrase = phrase.lower()

    # strip punctuation
    exclude = set(string.punctuation)
    phrase = ''.join(ch for ch in phrase if ch not in exclude)

    words = re.split('[\t ]', phrase)
    result = []
    for word in words:
        word = STOP_WORDS_RE.sub('', word)
        result.append(word)
    return ''.join(result)


def get_closest_match_index(word, possibilities):
    """Find the string that is most similar to the target string. Uses difflib's
    SequenceMatcher under the covers.

    :param word: The string to match to.
    :param possibilities: A list of possible matches.
    :return: The index of the closest matching possibility, if any. Else, return
    None.
    """
    word = normalize_phrase(word)
    possibilities = [normalize_phrase(x) for x in possibilities]
    try:
        match = difflib.get_close_matches(word, possibilities, n=1,
                                          cutoff=0.1)[0]
    except IndexError:
        # No good matches.
        return None
    return possibilities.index(match)
